package sim.util;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import sim.exception.SNoImplementationException;
import sim.exception.SRuntimeException;
import sim.math.SMath;

/**
 * La classe <b>SArrayutil</b> permet d'effectuer des oprations sur des tableaux (Array) de type primitif.
 * 
 * @author Simon Vzina
 * @since 2017-12-15
 * @version 2018-05-25 (Version labo 1.0 : Les fonctions discrtes)
 */
public final class SArrays {

  /**
   * Mthode pour dterminer si deux tableau de nombres de type double sont <b>relativement gaux</b>. 
   * En utilisant une approche de calcul de diffrence, on vrifie si pour chaque lment du tableau
   * <ul>a - b < EPSILON*ref</ul>  
   * afin de <b>valid l'galit</b> entre a et b (a == b). EPSILON est un seuil de prcision 
   * et ref est une base de rfrence (la valeur absolue la plus leve parmis a et b).
   * <p>Cenpendant, si les deux chiffres sont infrieurs  EPSILON, ils seront considrs comme gaux.</p>
   * 
   * @param tab1 Le premier tableau  comparer.
   * @param tab2 Le deuxime tableau  comparer.
   * @param epsilon La prcision acceptable.
   * @return <b>true</b> si les deux tableaux sont <b>relativement gaux</b> et <b>false</b> sinon.
   */
  public static boolean nearlyEquals(double[] tab1, double[] tab2, double epsilon)
  {
    // Vrifier que les deux tableaux ont la mme taille.
    if(tab1.length != tab2.length)
      return false;
    
    // Vrifier que l'ensemble des donne indexe par indexe sont "nearlyEquals".
    for(int i = 0; i < tab1.length; i++)
      if(!SMath.nearlyEquals(tab1[i], tab2[i], epsilon))
        return false;
    
    // Puisque l'ensemble des valeurs indexe par indexe sont "nearlyEquals", le tableau doit l'tre. 
    return true;
  }
  
  /**
   * Mthode pour effectuer une copie d'un tableau.
   * 
   * @param tab Le tableau  copier.
   * @param copy Le tableau qui va recevoir la copie.
   * @throws SRuntimeException Si les deux tableaux n'ont pas la mme longueur.
   */
  public static void copy(int[] tab, int[] copy) throws SRuntimeException
  {
    if(tab.length != copy.length)
      throw new SRuntimeException("Les deux tableaux n'ont pas la mme longueur (" + tab.length + " et " + copy + ").");
    
    for(int i = 0; i < tab.length; i++)
      copy[i] = tab[i];
  }
  
  /**
   * Mthode pour effectuer une copie d'un tableau.
   * 
   * @param tab Le tableau  copier.
   * @param copy Le tableau qui va recevoir la copie.
   * @throws SRuntimeException Si les deux tableaux n'ont pas la mme longueur.
   */
  public static void copy(int[] tab, byte[] copy) throws SRuntimeException
  {
    if(tab.length != copy.length)
      throw new SRuntimeException("Les deux tableaux n'ont pas la mme longueur (" + tab.length + " et " + copy + ").");
    
    for(int i = 0; i < tab.length; i++)
      copy[i] = (byte) tab[i];
  }
  
  /**
   * Mthode pour effectuer une copie d'un tableau.
   * 
   * @param tab Le tableau  copier.
   * @param copy Le tableau qui va recevoir la copie.
   * @throws SRuntimeException Si les deux tableaux n'ont pas la mme longueur.
   */
  public static void copy(byte[] tab, byte[] copy) throws SRuntimeException
  {
    if(tab.length != copy.length)
      throw new SRuntimeException("Les deux tableaux n'ont pas la mme longueur (" + tab.length + " et " + copy + ").");
    
    for(int i = 0; i < tab.length; i++)
      copy[i] = (byte) tab[i];
  }
  
  /**
   * Mthode pour effectuer une copie d'un tableau.
   * 
   * @param tab Le tableau  copier.
   * @param copy Le tableau qui va recevoir la copie.
   * @throws SRuntimeException Si les deux tableaux n'ont pas la mme longueur.
   */
  public static void copy(double[] tab, double[] copy) throws ArrayIndexOutOfBoundsException
  {
    if(tab.length != copy.length)
      throw new ArrayIndexOutOfBoundsException("Les deux tableaux n'ont pas la mme longueur (" + tab.length + " et " + copy + ").");
    
    for(int i = 0; i < tab.length; i++)
      copy[i] = tab[i];
  }
  
  /**
   * Mthode pour raliser <u>l'addition</u> d'une valeur  un tableau.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le 1er tableau.
   * @param a La valeur  ajouter.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] add(double[] tab, double a) 
  {
    double[] result = new double[tab.length];
    
    for(int i = 0; i < tab.length; i++)
      result[i] = tab[i] + a;
    
    return result;
  }
  
  /**
   * Mthode pour raliser <u>l'addition</u> entre deux tableaux de mme dimension.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab1 Le 1er tableau.
   * @param tab2 Le 2e tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   * @throws ArrayIndexOutOfBoundsException Si les deux tableaux n'ont pas la mme taille.
   */
  public static double[] add(double[] tab1, double[] tab2) throws ArrayIndexOutOfBoundsException
  {
    // Vrification de la taille des tableaux.
    if(tab1.length != tab2.length)
      throw new ArrayIndexOutOfBoundsException("Les deux tableaux n'ont pas la mme longueur (" + tab1.length + " et " + tab2 + ").");
    
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>la soustraction</u> d'une valeur  un tableau.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le 1er tableau.
   * @param a La valeur  soustraire.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] substract(double[] tab, double a) 
  {
    return add(tab, -1*a);
  }
  
  /**
   * Mthode pour raliser <u>la soustraction</u> entre deux tableaux de mme dimension.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab1 Le 1er tableau.
   * @param tab2 Le 2e tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   * @throws ArrayIndexOutOfBoundsException Si les deux tableaux n'ont pas la mme taille.
   */
  public static double[] substract(double[] tab1, double[] tab2) throws ArrayIndexOutOfBoundsException
  {
    if(tab1.length != tab2.length)
      throw new ArrayIndexOutOfBoundsException("Les deux tableaux n'ont pas la mme longueur (" + tab1.length + " et " + tab2 + ").");
    
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>la multiplication</u> d'une valeur  un tableau.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le 1er tableau.
   * @param a La valeur  multiplier.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] multiply(double[] tab, double a) 
  {
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>la multiplication</u> entre deux tableaux de mme dimension.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab1 Le 1er tableau.
   * @param tab2 Le 2e tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   * @throws ArrayIndexOutOfBoundsException Si les deux tableaux n'ont pas la mme taille.
   */
  public static double[] multiply(double[] tab1, double[] tab2) throws ArrayIndexOutOfBoundsException
  {
    if(tab1.length != tab2.length)
      throw new ArrayIndexOutOfBoundsException("Les deux tableaux n'ont pas la mme longueur (" + tab1.length + " et " + tab2 + ").");
    
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>le produit scalaire</u> entre deux tableaux de mme dimension.
   * 
   * @param A Le 1er tableau.
   * @param B Le 2e tableau.
   * @return Le rsultat de l'opration mathmatique.
   */
  public static double dot(double[] A, double[] B)
  {
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>la division</u> entre deux tableaux de mme dimension.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab1 Le 1er tableau.
   * @param tab2 Le 2e tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   * @throws ArrayIndexOutOfBoundsException Si les deux tableaux n'ont pas la mme taille.
   */
  public static double[] divide(double[] tab1, double[] tab2) throws ArrayIndexOutOfBoundsException
  {
    if(tab1.length != tab2.length)
      throw new ArrayIndexOutOfBoundsException("Les deux tableaux n'ont pas la mme longueur (" + tab1.length + " et " + tab2 + ").");
    
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>l'exponentiel</u> d'un tableau.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] exp(double[] tab)
  {
    double[] result = new double[tab.length];
    
    for(int i = 0; i < tab.length; i++)
      result[i] = Math.exp(tab[i]);
    
    return result;
  }
  
  /**
   * Mthode pour raliser <u>la puissance</u> d'un tableau.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le tableau.
   * @param n La puissance.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] pow(double[] tab, double n)
  {
    throw new SNoImplementationException("La mthode n'a pas t implmente.");
  }
  
  /**
   * Mthode pour raliser <u>le sinus</u> d'un tableau. Les valeurs du tableau seront considres en <b>radian</b>.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] sin(double[] tab)
  {
    double[] result = new double[tab.length];
    
    for(int i = 0; i < tab.length; i++)
      result[i] = Math.sin(tab[i]);
    
    return result;
  }
  
  /**
   * Mthode pour raliser <u>le cosinus</u> d'un tableau. Les valeurs du tableau seront considres en <b>radian</b>.
   * Un nouveau tableau sera retourn contenant le rsultat de l'opration mathmatique.
   * 
   * @param tab Le tableau.
   * @return Le tableau contenant le rsultat de l'opration mathmatique.
   */
  public static double[] cos(double[] tab)
  {
    double[] result = new double[tab.length];
    
    for(int i = 0; i < tab.length; i++)
      result[i] = Math.cos(tab[i]);
    
    return result;
  }
  
    
  /**
   * Mthode pour obtenir la <b>plus grande valeur</b> d'un tableau.
   * 
   * @param tab Le tableau.
   * @return La plus grande valeur du tableau.
   */
  public static int findMax(int[] tab)
  {
    int max = tab[0]; 
    
    for(int v : tab)
      if(v > max)
        max = v;
    
    return max;
  }
  
  /**
   * Mthode pour obtenir la <b>plus grande valeur</b> d'un tableau.
   * 
   * @param tab Le tableau.
   * @return La plus grande valeur du tableau.
   */
  public static double findMax(double[] tab)
  {
    double max = tab[0]; 
    
    for(double v : tab)
      if(v > max)
        max = v;
    
    return max;
  }
  
  /**
   * Mthode pour obtenir l'indice dans le tableau de la <b>valeur la plus grande</b>.
   * 
   * @param tab Le tableau.
   * @return L'indice dans le tableau o l'on trouve la valeur la plus grande.
   */
  public static int findIndexOfMax(double[] tab)
  {
    // Choisir la meilleure cote.
    int index = 0;
    double value = tab[index];
    
    // Parcourir le tableau  la recherche de la plus grande valeur.
    for(int i = 1; i < tab.length; i++)
      if(tab[i] > value)
      {
        value = tab[i];
        index = i;
      }
    
    return index;
  }
  
  /**
   * Mthode pour obtenir la <b>plus petite valeur</b> d'un tableau.
   * 
   * @param tab Le tableau.
   * @return La plus petite valeur du tableau.
   */
  public static int findMin(int[] tab)
  {
    int min = tab[0]; 
    
    for(int v : tab)
      if(v < min)
        min = v;
    
    return min;
  }
  
  /**
   * Mthode pour obtenir la <b>plus petite valeur</b> d'un tableau.
   * 
   * @param tab Le tableau.
   * @return La plus petite valeur du tableau.
   */
  public static double findMin(double[] tab)
  {
    double min = tab[0]; 
    
    for(double v : tab)
      if(v < min)
        min = v;
    
    return min;
  }
  
  /**
   * Mthode pour convertir un tableau d'entier vers un tableau de nombre rel entre une valeur minimale et maximale.
   * La correspondance entre les valeurs converties sera linaire.
   * 
   * @param data Les entiers  convertir.
   * @param min La borne minimale.
   * @param max La borne maximale.
   * @return Le tableau converti.
   * @throws SRuntimeException Si les bornes sont mal dfinies.
   */
  public static double[] mappingIntToDouble(int[] data, double min, double max) throws SRuntimeException
  {
    if(min > max)
      throw new SRuntimeException("Erreur SMath 003 : La borne minimale " + min + " et la borne maximale " + max + " sont mal dfinies.");
    
    int min_value = findMin(data);
    int max_value = findMax(data);
    
    double[] result = new double[data.length];
    
    // Itrer sur l'ensemble des lments du tableau.
    for(int i = 0; i < result.length; i++)
    {
      double t = SMath.reverseLinearInterpolation((double)data[i], (double)min_value, (double)max_value);
      result[i] = SMath.linearInterpolation(min, max, t);
    }
    
    return result;
  }
  
  /**
   * Mthode pour convertir un tableau de nombre rel vers un tableau d'entier entre une valeur minimale et maximale.
   * La correspondance entre les valeurs converties sera linaire.
   * 
   * @param data Les donnes  convertir.
   * @param min La borne minimale.
   * @param max La borne maximale.
   * @return Le tableau converti.
   * @throws IllegalArgumentException Si les bornes sont mal dfinies.
   */
  public static int[] mappingDoubleToInt(double[] data, int min, int max) throws IllegalArgumentException
  {
    return mappingDoubleToInt(data, findMin(data), findMax(data), min, max);
  }
  
  /**
   * ...
   * 
   * @param data
   * @param double_min
   * @param double_max
   * @param int_min
   * @param int_max
   * @return
   * @throws IllegalArgumentException
   */
  public static int[] mappingDoubleToInt(double[] data, double double_min, double double_max, int int_min, int int_max) throws IllegalArgumentException
  {
    if(int_min > int_max)
      throw new IllegalArgumentException("Erreur SArrayUtil 004 : La borne entire minimale " + int_min + " et la borne entire maximale " + int_max + " sont mal dfinies.");
    
    if(double_min > double_max)
      throw new IllegalArgumentException("Erreur SArrayUtil 004 : La borne double minimale " + double_min + " et la borne double maximale " + double_max + " sont mal dfinies.");
       
    int[] result = new int[data.length];
    
    // Itrer sur l'ensemble des lments du tableau.
    for(int i = 0; i < result.length; i++)
    {
      double t = SMath.reverseLinearInterpolation(data[i], double_min, double_max);
      result[i] = (int)SMath.linearInterpolation(int_min, int_max, t);
    }
    
    return result;
  }
  
  /**
   * <p>
   * Mthode permettant de gnrer un tableau contenant les lments identique de deux tableaux de valeur <ul>pralablement tri</ul>.
   * </p>
   * 
   * <p>
   * <b>REMARQUE</b> : Le fonctionnement de cette mthode ne sera pas valide si les deux tableaux passs en paramtre ne sont pas palablement tri.
   * </p>
   * Rfrence : https://stackoverflow.com/questions/32676381/find-intersection-of-two-arrays
   * 
   * @param tab1 Le 1ier tableau  comparer.
   * @param tab2 Le 2ime tableau  comparer.
   * @return Un tableau comprenant les lments identiques de deux tableaux (l'intersection des deux tableaux).
   */
  public static double[] intersectionSortedArray(double[] tab1, double[] tab2)
  {
    return intersectionSortedArray(tab1, tab2, SMath.EPSILON);
  }
  
  /**
   * <p>
   * Mthode permettant de gnrer un tableau contenant les lments identique de deux tableaux de valeur <ul>pralablement tri</ul>.
   * </p>
   * 
   * <p>
   * <b>REMARQUE</b> : Le fonctionnement de cette mthode ne sera pas valide si les deux tableaux passs en paramtre ne sont pas palablement tri.
   * </p>
   * Rfrence : https://stackoverflow.com/questions/32676381/find-intersection-of-two-arrays
   * 
   * @param tab1 Le 1ier tableau  comparer.
   * @param tab2 Le 2ime tableau  comparer.
   * @param epsilon La prcision de la comparaison.
   * @return Un tableau comprenant les lments identiques de deux tableaux (l'intersection des deux tableaux).
   */
  public static double[] intersectionSortedArray(double[] tab1, double[] tab2, double epsilon)
  {
    double intersection[] = new double[Math.min(tab1.length, tab2.length)];
    int count = 0;

    int i = 0; int j = 0;
    while (i < tab1.length && j < tab2.length)
    {
      // Vrifier s'il y a galit.
      if(SMath.nearlyEquals(tab1[i], tab2[j], epsilon)) 
      {
        intersection[count] = tab1[i];
        count++;
        i++;
        j++;
      }
      else
        // Avancer dans la recherche des lments.
        if (tab1[i] < tab2[j]) 
          i++;                    // avancer dans le tableau 1
        else
          j++;                    // avancer dans le tableau 2
    }

    // Construire un tableau avec l'espace mmoire minimum.
    double[] result = new double[count];
    for(int k = 0; k < result.length; k++)
      result[k] = intersection[k];
    
    return result;
  }
  
  /**
   * Mthode pour changer la dimension d'un tableau. Si la nouvelle taille est infrieure  la prcdente, les derniers lments ne seront pas copi.
   * Si la nouvelle taille est suprieure  la prcdente, des valeurs nulles sont attribues aux positions suprieures. 
   * 
   * @param array Le tableau  copier.
   * @param new_size La nouvelle dimension du tableau.
   * @return Un tableau avec une copie des donnes avec une taille diffrente.
   */
  public static double[] resize(double[] array, int new_size)
  {
      // Remplir un nouveau tableau ayant seulement la taille demand.
      // Tous les lments du tableau prcdent 
      double[] result = new double[new_size];
      
      for(int i = 0; i < new_size; i++)
        result[i] = array[i];
     
      return result;
  }
  
  /**
   * Mthode pour changer la dimension d'un tableau. Si la nouvelle taille est infrieure  la prcdente, les derniers lments ne seront pas copi.
   * Si la nouvelle taille est suprieure  la prcdente, des valeurs nulles sont attribues aux positions suprieures. 
   * 
   * @param array Le tableau  copier.
   * @param new_size La nouvelle dimension du tableau.
   * @return Un tableau avec une copie des donnes avec une taille diffrente.
   */
  public static int[] resize(int[] array, int new_size)
  {
      // Remplir un nouveau tableau ayant seulement la taille demand.
      // Tous les lments du tableau prcdent 
      int[] result = new int[new_size];
      
      for(int i = 0; i < new_size; i++)
        result[i] = array[i];
     
      return result;
  }
  
  /**
   * Mthode pour faire l'criture d'un tableau dans un fichier texte.
   * 
   * @param file_name Le nom du fichier.
   * @param values Le tableau des valeurs  crire.
   */
  public static void write(String file_name, double[] values)
  {
    try{
      
      FileWriter fw = new FileWriter(file_name);
      BufferedWriter bw = new BufferedWriter(fw);
    
      // Faire l'criture de chaque valeur et changer de ligne.
      for(int i = 0; i < values.length; i++)
      {
        bw.write(Double.toString(values[i]));
        bw.newLine();
      }
      
      bw.close(); //  fermer celui-ci en premier, sinon, ERROR !!!
      fw.close();   
      
    }catch(IOException ioe){
      SLog.logWriteLine("Message SArrays - Une erreur de type I/O est survenue.");
      ioe.printStackTrace();
    }
  }
  
}
